#include "html_string.hpp"
#include <cstddef>
#include <cstdlib>
#include <exception>
#include <format>
#include <iostream>
#include <ostream>
#include <sstream>
#include <stdexcept>
#include <string>
#include <utility>
#include <vector>
#include <webview/backends.hh>
#include <webview/webview.h>

extern char** environ;

using vpss_t = std::vector<std::pair<std::string, std::string>>;
vpss_t getEnvKv(){
    vpss_t result;
    for(char **env = environ; *env != nullptr; env++){
        const std::string thisEnv = *env;
        const size_t idx = thisEnv.find('=');
        if (idx == std::string::npos) {
            throw std::logic_error("cannot parse env");
        }
        result.emplace_back(thisEnv.substr(0,idx),thisEnv.substr(idx+1));
    }
    return result;
}

template<class T,class U>
std::ostream&operator<<(std::ostream &os,const std::pair<T,U> &p){
    os<<"{ "<<p.first<<", "<<p.second<<" }";
    return os;
}

template<class T>
std::ostream&operator<<(std::ostream &os,const std::vector<T> &v){
    os<<"[ ";
    if(v.size()==1){
        os<<v[0]<<" ]";
        return os;
    }
    for(size_t i=0;i<v.size()-1;i++){
        os<<v[i]<<", ";
    }
    os<<" ]";
    return os;
}



int main(const int argc,char *argv[],char *env[]){
    try {
        std::cout<<getEnvKv()<<'\n';
        webview::webview wv(true,nullptr);
        wv.set_html(html);
        wv.bind("getEnvString",
            [](const std::string &req)->std::string {
                std::stringstream oss;
                oss<<getEnvKv();
                std::cout<<std::format("{{env:\"{}\"}}",getEnvKv()[0].first)<<'\n';
                return std::format("{{\"env\":\"{}\"}}",getEnvKv()[0].first);
            });
        wv.run();
    } catch (const std::exception &e) {
        std::cerr<<e.what()<<'\n';
        return 1;
    }
}